// Aspell word list conversion functions
// Copyright 1998 by Kevin Atkinson under the terms of the LGPL

#ifndef __as_convert__
#define __as_convert__

#include "as_error.hh"
#include "as_language.hh"

class SCC_WordList_Internal;

struct SCC_WordList {
  SCC_WordList_Internal* internal;

  SCC_WordList();
  ~SCC_WordList();
  bool add(const string &str);
};

void del (const SC_Language* lang);
const SC_Error create_master2 (const string &base, unsigned int size, 
			       SCC_WordList &wl, const SC_Language *lang);
const SC_Error create_personal2(const string &base, 
				SCC_WordList &wl,
				const SC_Language *lang);

template <class InputIterator>
const SC_Error create_master(const string base, 
			     const InputIterator begin, 
			     const InputIterator end,
			     const string &lang0 = "english") {
  const SC_Language *lang = SC_Languages(lang0.c_str());
  if (!lang) return SC_Error("", lang0, SC_Error::unknown_lang);
  SC_Error error = create_master(base, begin, end, lang);
  del(lang);
  return error;
}

template <class InputIterator>
const SC_Error create_master(const string base, 
			     const InputIterator begin, 
			     const InputIterator end,
			     const SC_Language *lang) {
  SCC_WordList word_list;
  
  unsigned int size = 0;

  for (InputIterator i = begin; i != end; ++i) {
    if (word_list.add(*i)) {
      size += (*i).size() + 1;
    }
  }
  return create_master2(base, size, word_list, lang);
}

template <class InputIterator>
const SC_Error create_personal(const string base,
			       const InputIterator begin,
			       const InputIterator end,
			       const string &lang0 = "english") {
  const SC_Language *lang = SC_Languages(lang0.c_str());
  if (!lang) return SC_Error("", lang0, SC_Error::unknown_lang);
  SC_Error error = create_personal(base, begin, end, lang);
  del(lang);
  return error;
}
 
template <class InputIterator>
const SC_Error create_personal(const string base,
			       const InputIterator begin,
			       const InputIterator end,
			       const SC_Language *lang) {
  SCC_WordList word_list;
  
  for (InputIterator i = begin; i != end; ++i) {
    word_list.add(*i);
  }
  
  return create_personal2(base, word_list, lang);
}

#endif

