#ifndef as_mwordlist_hh__
#define as_mwordlist_hh__

#include "const_string.hh"

class SC_MWordList;
class SCMWL_Internal;
class SC_Soundslike;
class aspell;

class SC_MWordList_Iterator {
  friend class SC_MWordList;
public:
  typedef const_string   value_type;
  typedef unsigned int   size_type;
  typedef          int   difference_type;

private:
  enum which_type {main,personal,session,end};

private:  
  const SCMWL_Internal *internal;
  which_type           which;
  size_type            offset;

private:
  SC_MWordList_Iterator(const SCMWL_Internal*);       // begin
  SC_MWordList_Iterator(const SCMWL_Internal*, bool); // end
  void advance_past_nulls();

public:
  const value_type operator*() const;
  SC_MWordList_Iterator& operator++();
  const SC_MWordList_Iterator operator++ (int);

  inline friend bool operator== (const SC_MWordList_Iterator& rhs, 
				 const SC_MWordList_Iterator& lhs)
    {return rhs.which==lhs.which && rhs.offset==lhs.offset;}

  inline friend bool operator!= (const SC_MWordList_Iterator& rhs,
				 const SC_MWordList_Iterator& lhs)
    {return !(rhs==lhs);}
};

class SC_MWordList {
private:
  const SCMWL_Internal *internal;
public:
  SC_MWordList(const SCMWL_Internal&);
  typedef SC_MWordList_Iterator::value_type      value_type;
  typedef SC_MWordList_Iterator::size_type       size_type;
  typedef SC_MWordList_Iterator::difference_type difference_type;
  typedef SC_MWordList_Iterator iterator;
  typedef SC_MWordList_Iterator const_iterator;

  SC_MWordList();
  SC_MWordList(const SC_MWordList&);
  SC_MWordList& operator=(const SC_MWordList&);
  ~SC_MWordList();

  size_type      size()  const;
  bool           empty() const {return !size();}
  const_iterator begin() const {return const_iterator(internal);}
  const_iterator end()   const {return const_iterator(internal, true);}
};
 

#endif
